from onegov.winterthur import _

# translations provided by onegov.org, overriden by Winterthur
_("Hide from the public")
_(
    "This site is hidden from the general public. "
    "To show it to everyone, edit this page and disable \"Hide from public\"."
)
